/*
 * FILE NAME ralink_gpio_cp.h
 *
 * BRIEF MODULE DESCRIPTION
 *  Driver for Ralink RT3XXX GPIO, with different functionality than Ralink's version.
 *
 *  Author: CradlePoint Technology, Inc.  <source@cradlepoint.com>
 *          Steve Johnson <stevej@cradlepoint.com>
 *
 *  Derived from Ralink's ralink_gpio.h
 *  Copyright, Ralink Technology, Inc.
 *
 *  Copyright 2012 CradlePoint Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2
 *  as published by the Free Software Foundation.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE	LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __RALINK_GPIO_H__
#define __RALINK_GPIO_H__

/*
 * ioctl commands
 */
#define	RALINK_GPIO_READ_BIT		_IOR('G', 0x01, int)
#define	RALINK_GPIO_SET_BIT			_IOW('G', 0x02, int)
#define	RALINK_GPIO_CLEAR_BIT		_IOW('G', 0x03, int)
#define	RALINK_GPIO_BLINK		_IOW('G', 0x04, int)
#define	RALINK_GPIO_NO_BLINK		_IOW('G', 0x05, int)
#define	RALINK_GPIO_SERIAL_ENABLE	_IOW('G', 0x08, int)
#define	RALINK_GPIO_SERIAL_DISABLE	_IOW('G', 0x09, int)
#define	RALINK_GPIO_READ_DIR_BIT	_IOR('G', 0x0a, int)
#define	RALINK_GPIO_CLEAR_DIR_BIT	_IOW('G', 0x0b, int)
#define	RALINK_GPIO_SET_DIR_BIT		_IOW('G', 0x0c, int)
struct ralink_gpio_reg_t {
	unsigned int max;
	unsigned int data;
	unsigned int dir;
	unsigned int set;
	unsigned int clear;
	unsigned int interrupt;
	unsigned int int_edge;
	unsigned int int_rena;
	unsigned int int_fena;
	unsigned int intp_value;
	unsigned int rena_value;
	unsigned int fena_value;
};

struct ralink_gpio_reg_t ralink_gpio_reg[] = {
	{24, RALINK_REG_PIODATA, RALINK_REG_PIODIR, RALINK_REG_PIOSET, RALINK_REG_PIORESET, 
	 RALINK_REG_PIOINT, RALINK_REG_PIOEDGE, RALINK_REG_PIORENA, RALINK_REG_PIOFENA, 0, 0, 0},
	{40, RALINK_REG_PIO3924DATA, RALINK_REG_PIO3924DIR, RALINK_REG_PIO3924SET, RALINK_REG_PIO3924RESET,
	 RALINK_REG_PIO3924INT, RALINK_REG_PIO3924EDGE, RALINK_REG_PIO3924RENA, RALINK_REG_PIO3924FENA, 0, 0, 0},
	{72, RALINK_REG_PIO7140DATA, RALINK_REG_PIO7140DIR, RALINK_REG_PIO7140SET, RALINK_REG_PIO7140RESET,
	 RALINK_REG_PIO7140INT, RALINK_REG_PIO7140EDGE, RALINK_REG_PIO7140RENA, RALINK_REG_PIO7140FENA, 0, 0, 0}
#if defined(CONFIG_RALINK_RT3883) || defined(CONFIG_RALINK_MT7620)
	, {96, RALINK_REG_PIO9572DATA, RALINK_REG_PIO9572DIR, RALINK_REG_PIO9572SET, RALINK_REG_PIO9572RESET,
	   RALINK_REG_PIO9572INT, RALINK_REG_PIO9572EDGE, RALINK_REG_PIO9572RENA, RALINK_REG_PIO9572FENA, 0, 0, 0}
#endif
};

// Special commands to pass data to bootloader,
//  masquerading as GPIOs
#define BOOT_COUNT (GPIO_PINS)
#define UPGRADE (GPIO_PINS + 1)
#define SPECIAL_COMMANDS 2

#if defined(CONFIG_RALINK_RT3883) || defined(CONFIG_RALINK_MT7620)
#define LAST_REG 0x6b0
#else
#define LAST_REG 0x680
#endif

/*
  I had this really cool plan to put all this into platform code and pass it to the driver
  when I realized this is so platform-specific that I might as well just put it along with
  the driver.  It's not like anyone else needs a GPIO module that listens to interrupts
  and sets GPIOs and sends events.
*/

#if defined(PRODUCT_CONGO)

#define THERMAL_IC                17
#define CGPIO_CONNECTOR_INPUT     66    // input, interrupt
#define CGPIO_SERIAL_INPUT_1      9
#define CGPIO_SERIAL_INPUT_2      10
#define CGPIO_SERIAL_INPUT_3      13
#define USB_POWER_FAIL            67
#define SOFT_RST_IN_BUTTON        71    // input, interrupt

#define DEBOUNCED_PINS 7

#endif

#if defined(PRODUCT_RUBY)

#define SATA_GPIO1                7
#define SATA_GPIO2                8
#define SATA_GPIO3                9
#define SATA_GPIO4                10
#define SATA_IGNITION_SENSE       11
#define MINICARD_OVC              17    // input, interrupt
#define USB_POWER_CTRL            18
#define MINICARD_POWER_CTRL       19
#define CONNECTOR_INPUT           66    // input, interrupt
#define USB_OVC                   67    // input, interrupt
#define SOFT_RST_IN_BUTTON        71    // input, interrupt

#define DEBOUNCED_PINS 9 

#endif

#if defined(PRODUCT_IBR200)

#define USB_POWER_CTRL            7
#define USB_OVC                   9    // input, interrupt
#define MINICARD_POWER_CTRL       19
#define SIM_DOOR_DETECT           65    // input dedge
#define CONNECTOR_INPUT           66    // input, interrupt
#define MINICARD_OVC              67    // input, interrupt
#define SOFT_RST_IN_BUTTON        71    // input, interrupt

#define DEBOUNCED_PINS            5

#endif

#if defined(PRODUCT_CBA550)

#define MINICARD_POWER_CTRL       61    // output
#define SIM_DOOR_DETECT           65    // input dedge
#define SOFT_RST_IN_BUTTON        71    // input, interrupt

#define DEBOUNCED_PINS            2

#endif

// Kernel Events
//  These are platform-neutral
#define WPS_BUTTON_EVT 1
#define RESET_BUTTON_EVT 2
#define POWER_BUTTON_EVT 3
#define IN_5V_EVT 4
//#define PWR_FLAG_3G_EVT 5
#define DOCK_SENSE_EVT 6
#define LAN_WAN_SW_EVT 7
#define WIFI_ENABLE_EVT 8
#define SS_BUTTON_EVT 9
#define CURRENT_LIMIT_EVT 10
#define EXCARD_ATTACH_EVT 11
#define CONFIGURABLE_GPIO_EVT 12
#define THERMAL_EVT 13
#define USB_POWER_EVT 14
#define SIM_DOOR_EVT 15

// Platform-specific GPIO/Event bindings

// - interrupt is the received GPIO interrupt pin
// - event is the event to send based on the interrupt
// - disable is a GPIO to turn off based on the
//  interrupt.  With a USB overcurrent we receive
//  an overcurrent event and then cut the power
//  on the USB device in the interrupt handler.
struct gpio_event_t {
	u8 gpio_interrupt;
	u8 gpio_event;
	u8 gpio_disable;
	u8 gpio_event_occurred;
};

#if defined(PRODUCT_CONGO)
static struct gpio_event_t ralink_gpio_event[DEBOUNCED_PINS] = {
	{ THERMAL_IC, THERMAL_EVT, 0, 0 },
	{ CGPIO_CONNECTOR_INPUT, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ CGPIO_SERIAL_INPUT_1, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ CGPIO_SERIAL_INPUT_2, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ CGPIO_SERIAL_INPUT_3, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ SOFT_RST_IN_BUTTON, RESET_BUTTON_EVT, 0, 0 },
	{ USB_POWER_FAIL, CURRENT_LIMIT_EVT, 0, 0 },
};
#endif

#if defined(PRODUCT_IBR200)
static struct gpio_event_t ralink_gpio_event[DEBOUNCED_PINS] = {
	{ MINICARD_OVC, CURRENT_LIMIT_EVT, MINICARD_POWER_CTRL, 0 },
	{ CONNECTOR_INPUT, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ USB_OVC, CURRENT_LIMIT_EVT, USB_POWER_CTRL, 0 },
	{ SIM_DOOR_DETECT, SIM_DOOR_EVT, 0, 0 },
	{ SOFT_RST_IN_BUTTON, RESET_BUTTON_EVT, 0, 0 },
};
#endif

#if defined(PRODUCT_CBA550)
static struct gpio_event_t ralink_gpio_event[DEBOUNCED_PINS] = {
	{ SIM_DOOR_DETECT, SIM_DOOR_EVT, 0, 0 },
	{ SOFT_RST_IN_BUTTON, RESET_BUTTON_EVT, 0, 0 },
};
#endif

#if defined(PRODUCT_RUBY)
static struct gpio_event_t ralink_gpio_event[DEBOUNCED_PINS] = {
	{ MINICARD_OVC, CURRENT_LIMIT_EVT, MINICARD_POWER_CTRL, 0 },
	{ CONNECTOR_INPUT, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ SATA_GPIO1, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ SATA_GPIO2, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ SATA_GPIO3, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ SATA_GPIO4, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ SATA_IGNITION_SENSE, CONFIGURABLE_GPIO_EVT, 0, 0 },
	{ USB_OVC, CURRENT_LIMIT_EVT, USB_POWER_CTRL, 0 },
	{ SOFT_RST_IN_BUTTON, RESET_BUTTON_EVT, 0, 0 },
};
#endif

#endif // __RALINK_GPIO_H__
